@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -111,6 +111,9 @@
     do {} while (0)
 #endif
 
+#if !CONFIG_ERROR_FUNCTION_ATTRIBUTE
+#error we need GCC to support the 'error' function attribute
+#else
 #define STATIC(E)                                   \
     ({                                              \
         extern int __attribute__                    \
@@ -118,6 +121,7 @@
             static_assert();                        \
         (void) ((E) ? 0 : static_assert());         \
     })
+#endif
 
 void fatal_error(const char* fmt, ...)
     PRINTF(1)
@@ -167,8 +171,5 @@ size_t json_timeval_subtract(
 size_t json_timeval_to_usecs(
     const struct timeval*);
 
-bool json_validate_utf8(
-    const uchar_t* buf, size_t len, size_t* err);
-
 #endif/*__COMMON_H*/
 
