@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -532,7 +532,7 @@ enum json_parse_status_t json_ast_parse_done(
         return json_parse_status_ok;
 
     c = json_parse_get_config(ast->json_obj);
-    m = c & (1 << json_allow_multi_objects_config);
+    m = c & (SZ(1) << json_allow_multi_objects_config);
     n = JSON_AST_STACK_SIZE();
 
     if ((!m && n != 1) || (m && n == 0))
