@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -21,6 +21,7 @@
 #include <stdio.h>
 
 #include "json-defs.h"
+#include "va-args.h"
 
 enum {
     pretty_print_char_quotes   = 1U << 0,
@@ -36,16 +37,32 @@ JSON_API size_t pretty_print_string(
     FILE* file, const uchar_t* str, size_t len, size_t flags);
 
 enum pretty_print_repr_flags_t {
-    pretty_print_repr_quote_non_print     = 1U << 0,
-    pretty_print_repr_quote_non_ascii     = 1U << 1,
-    pretty_print_repr_unicode_quote_cntrl = 1U << 2,
-    pretty_print_repr_print_quotes        = 1U << 3,
+    pretty_print_repr_quote_non_print = 1U << 0,
+    pretty_print_repr_print_quotes    = 1U << 1,
+    pretty_print_repr_escape_utf8     = 1U << 2,
+    pretty_print_repr_surrogate_pairs = 1U << 3,
 };
 
 JSON_API void pretty_print_repr(
     FILE* file, const uchar_t* str, size_t len,
     enum pretty_print_repr_flags_t flags);
 
+#define PRETTY_PRINT_REPR_FLAGS_(n) \
+    pretty_print_repr_ ## n
+#define PRETTY_PRINT_REPR_FLAGS(...) \
+    (VA_ARGS_REPEAT(|, PRETTY_PRINT_REPR_FLAGS_, __VA_ARGS__))
+
+#define PRETTY_PRINT_REPR_FLAGS_IS__(n) \
+    (flags & PRETTY_PRINT_REPR_FLAGS_(n))
+#define PRETTY_PRINT_REPR_FLAGS_IS_(o, ...) \
+    (VA_ARGS_REPEAT(o, PRETTY_PRINT_REPR_FLAGS_IS__, __VA_ARGS__))
+#define PRETTY_PRINT_REPR_FLAGS_IS_OR(...) \
+    PRETTY_PRINT_REPR_FLAGS_IS_(||, __VA_ARGS__)
+#define PRETTY_PRINT_REPR_FLAGS_IS_AND(...) \
+    PRETTY_PRINT_REPR_FLAGS_IS_(&&, __VA_ARGS__)
+#define PRETTY_PRINT_REPR_FLAGS_IS \
+    PRETTY_PRINT_REPR_FLAGS_IS_AND
+
 #endif/*__PRETTY_PRINT_H*/
 
 
