@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -23,15 +23,21 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#include <signal.h>
+#include <unistd.h>
 #include <errno.h>
 #include <getopt.h>
 #include <limits.h>
 #include <ctype.h>
+#include <locale.h>
+#include <time.h>
+#include <link.h>
 
 #include "lib/json.h"
 #include "lib/pretty-print.h"
 
 #include "common.h"
+#include "char-traits.h"
 
 #ifdef DEBUG
 #define PRINT_DEBUG_COND true
@@ -41,12 +47,15 @@
 #define STRINGIFY_(s) #s
 #define STRINGIFY(s)  STRINGIFY_(s)
 
+#define ISASCII CHAR_IS_ASCII
+#define ISDIGIT CHAR_IS_DIGIT
+
 static const char program[] = STRINGIFY(PROGRAM);
 static const char library[] = STRINGIFY(LIBRARY);
-static const char verdate[] = "0.8.7 -- 2017-11-14 09:09"; // $ date +'%F %R'
+static const char verdate[] = "0.8.8 -- 2017-12-02 10:12"; // $ date +'%F %R'
 
 static const char license[] =
-"Copyright (C) 2016, 2017  Stefan Vargyas.\n"
+"Copyright (C) 2016, 2017, 2018  Stefan Vargyas.\n"
 "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n"
 "This is free software: you are free to change and redistribute it.\n"
 "There is NO WARRANTY, to the extent permitted by law.\n";
@@ -272,21 +281,90 @@ bool mem_find_nth(
     return false;
 }
 
+static void sigpipe_no_error(
+    int signal UNUSED)
+{
+    _exit(0);
+}
+
+static void init_sigpipe_action(
+    enum options_sigpipe_act_t act)
+{
+    static void (* const acts[])(int) = {
+        [options_sigpipe_act_error]    = SIG_DFL,
+        [options_sigpipe_act_no_error] = sigpipe_no_error,
+        [options_sigpipe_act_ignore]   = SIG_IGN,
+    };
+    void (* const* f)(int);
+    struct sigaction a;
+
+    f = ARRAY_NULL_ELEM_REF(acts, act);
+    VERIFY(f != NULL);
+
+    memset(&a, 0, sizeof a);
+    a.sa_handler = *f;
+
+    sigaction(SIGPIPE, &a, NULL);
+}
+
+static int dl_callback(
+    struct dl_phdr_info* info,
+    size_t size UNUSED, void* result)
+{
+    const char **r = result, *p;
+
+    if (info->dlpi_name &&
+        (p = strrchr(info->dlpi_name, '/')) &&
+        !strcmp(p + 1, library)) {
+        *r = info->dlpi_name;
+        return 1;
+    }
+    return 0;
+}
+
+static char* lib_runtime_path(void)
+{
+    const char* r = NULL;
+    char *p, *q;
+
+    if (!dl_iterate_phdr(dl_callback, &r))
+        return NULL;
+    ASSERT(r != NULL);
+
+    if (!(p = realpath(r, NULL)))
+        return NULL;
+
+    if (!(q = strrchr(p, '/'))) {
+        free(p);
+        return NULL;
+    }
+
+    if (q == p) q ++;
+    *q = 0;
+
+    return p;
+}
+
 struct lib_version_t
 {
     unsigned short major;
     unsigned short minor;
     unsigned short patch;
-    const char*    build;
+    char*          build;
+    char*          rpath;
 };
 
-static struct lib_version_t lib_version(void)
+static struct lib_version_t lib_version(
+    bool extra_info)
 {
-    enum { M = 10000, m = 100 };
+    enum { M = 10000, m = 100, d = 20 };
     struct lib_version_t r;
-    const char* b;
+    const char *b, *c;
+    struct tm t;
     size_t v;
 
+    memset(&r, 0, sizeof(r));
+
     v = json_version();
     ASSERT(v > 0 && v < M * 10);
 
@@ -295,18 +373,46 @@ static struct lib_version_t lib_version(void)
     r.minor = v / m;
     r.patch = v % m;
 
+    if (!extra_info)
+        return r;
+
     b = json_build_datetime();
     ASSERT(b != NULL);
-    r.build = b;
+
+    c = setlocale(LC_TIME, "en_US");
+    ASSERT(c != NULL);
+
+    memset(&t, 0, sizeof(t));
+    if (!strptime(b, "%b %d %Y %H:%M:%S", &t))
+        fatal_error("failed parsing library build date");
+
+    c = setlocale(LC_TIME, c);
+    ASSERT(c != NULL);
+
+    r.build = malloc(d);
+    if (r.build == NULL)
+        OOM_ERROR("malloc failed creating "
+            "lib_version_t::build");
+
+    v = strftime(r.build, d, "%Y-%m-%d %H:%M:%S", &t);
+    ASSERT(v);
+
+    r.rpath = lib_runtime_path();
 
     return r;
 }
 
+static void lib_version_done(struct lib_version_t* ver)
+{
+    free(ver->rpath);
+    free(ver->build);
+}
+
 static void lib_version_check(void)
 {
     struct lib_version_t lib;
 
-    lib = lib_version();
+    lib = lib_version(false);
     if (lib.major == JSON_VERSION_MAJOR &&
         lib.minor == JSON_VERSION_MINOR)
         return;
@@ -328,15 +434,19 @@ static void version(void)
 {
     struct lib_version_t v;
 
-    v = lib_version();
+    v = lib_version(true);
+
     fprintf(stdout, 
         "%s: version %s\n"
         "%s: %s: version: %u.%u.%u\n"
         "%s: %s: build: %s\n"
+        "%s: %s: path: %s\n"
         "%s: %s: JSON_DEBUG is %sabled\n\n%s",
         program, verdate,
         program, library, v.major, v.minor, v.patch,
         program, library, v.build,
+        program, library, v.rpath
+            ? v.rpath : "???",
         program, library,
 #ifdef JSON_DEBUG
         "en",
@@ -345,6 +455,8 @@ static void version(void)
 #endif
         license
     );
+
+    lib_version_done(&v);
 }
 
 #define KB(X) (SZ(X) * SZ(1024))
@@ -489,29 +601,39 @@ static void usage(void)
         "                                 when action is anything but `-S|--escape-json';\n"
         "                                 when action is `-S|--escape-json', produce\n"
         "                                 surrogate pairs on output (default do not)\n"
-        "                                 when given `-u|--validate-utf8' too\n"
+        "                                 when given `-u|--validate-utf8' too; given along\n"
+        "                                 with `-c|--escape-utf8', generates surrogate pairs\n"
+        "                                 in the strings produced by `-P|--pretty-print',\n"
+        "                                 `-R|--terse-print', `Y|--type-print',\n"
+        "                                 `-A|--ast-print' or `F|--from-ast-print'\n"
         "  -u|--[no-]valid[ate]-utf8    validate for UTF8 encoding the non-ascii chars\n"
         "                                 in JSON strings (when the action given is any but\n"
         "                                 `-S|--escape-json') or in whole text (when given\n"
         "                                 `-S|--escape-json'); default: do not validate\n"
+        "  -c|--[no-]escape-utf8        escape UTF8 byte sequences or do not (default not)\n"
+        "                                 in the strings produced by `-P|--pretty-print',\n"
+        "                                 `-R|--terse-print', `Y|--type-print',\n"
+        "                                 `-A|--ast-print' or `F|--from-ast-print'\n"
         "  -a|--[no-]ascii-only         enforce JSON strings to contain ASCII chars\n"
         "                                 only or otherwise do not (default not)\n"
         "  -m|--[no-]multi-object       allow multiple JSON objects on input or\n"
         "                                 otherwise do not (default not)\n"
         "  -o|--[no-]print-dots         put indenting dots in structure print outs\n"
         "                                 or otherwise do not (default not)\n"
-        "  -q|--[no-]quote-control      print out quoted or plain ASCII control chars\n"
-        "                                 in the strings produced by `-P|--pretty-print',\n"
-        "                                 `-R|--terse-print', `Y|--type-print',\n"
-        "                                 `-A|--ast-print' or `F|--from-ast-print'\n"
-        "                                 (default: plain form); the chars BS, FF, LF,\n"
-        "                                 CR and HT are always printed in quoted form\n"
+        "  -p|--sigpipe-action NAME     specify program's behavior when receiving SIGPIPE;\n"
+        "                                 NAME can be any of: 'error' or 'no-error' for\n"
+        "                                 terminating the execution with or without error,\n"
+        "                                 or 'ignore' for ignoring the signal altogether\n"
+        "                                 (the default is 'error'); option `-p' cuts short\n"
+        "                                 the long option `--sigpipe-action=no-error'\n"
         "  -Q|--[no-]quote-text         print out double quote chars surrounding the\n"
         "                                 text produced by action `-S|--escape-json'\n"
         "                                 or otherwise do not (default not)\n"
         "  -r|--[no-]trim-spaces        trim out heading and trailing space chars in the\n"
         "                                 input text when action is `-S|--escape-json'\n"
         "                                 or otherwise do not (default not)\n"
+        "  -y|--[no-]empty-input        allow empty input when JSON text expected or\n"
+        "                                 otherwise do not (default not)\n"
         "     --[no-]newline-sep        separate the top values on terse JSON print outs\n"
         "                                 by newline or otherwise do not (default not)\n"
         "     --[no-]error              print out error reports on stderr or otherwise\n"
@@ -767,6 +889,15 @@ static void dump_options(const struct options_t* opts)
         CASE(def),
         CASE(lib)
     };
+#undef  CASE
+#undef  CASE2
+#define CASE2(n, s) [options_sigpipe_act_ ## n] = s
+#define CASE(n)     CASE2(n, #n)
+    static const char* const sigpipe_acts[] = {
+        CASE(error),
+        CASE2(no_error, "no-error"),
+        CASE(ignore)
+    };
     static const char* const noyes[] = {
         [0] = "no",
         [1] = "yes"
@@ -788,10 +919,12 @@ static void dump_options(const struct options_t* opts)
         "type-def:                      %s\n"
         "type-lib:                      %s\n"
         "type-name:                     %s\n"
-        "quote-cntrl:                   %s\n"
+        "sigpipe-act:                   %s\n"
+        "empty-input:                   %s\n"
         "liter-value:                   %s\n"
         "surrogate-pairs:               %s\n"
         "valid-utf8:                    %s\n"
+        "escape-utf8:                   %s\n"
         "ascii-only:                    %s\n"
         "multi-obj:                     %s\n"
         "print-dots:                    %s\n"
@@ -832,10 +965,12 @@ static void dump_options(const struct options_t* opts)
         NNUL(type_def),
         NNUL(type_lib),
         NNUL(type_name),
-        NOYES(quote_cntrl),
+        NAME(sigpipe_act),
+        NOYES(empty_input),
         NOYES(liter_value),
         NOYES(surrogate_pairs),
         NOYES(valid_utf8),
+        NOYES(escape_utf8),
         NOYES(ascii_only),
         NOYES(multi_obj),
         NOYES(print_dots),
@@ -1072,10 +1207,69 @@ static enum options_typelib_func_t parse_typelib_func_optarg(
     return r;
 }
 
+// $ print 'error =options_sigpipe_act_error' 'no-error =options_sigpipe_act_no_error' 'ignore =options_sigpipe_act_ignore'|gen-func -f lookup_sigpipe_act_name|ssed -R '1s/^/static /;1s/\s*result_t\&/\n\tenum options_sigpipe_act_t*/;s/(?=t =)/*/;s/result_t:://'
+
+static bool lookup_sigpipe_act_name(const char* n,
+    enum options_sigpipe_act_t* t)
+{
+    // pattern: error|ignore|no-error
+    switch (*n ++) {
+    case 'e':
+        if (*n ++ == 'r' &&
+            *n ++ == 'r' &&
+            *n ++ == 'o' &&
+            *n ++ == 'r' &&
+            *n == 0) {
+            *t = options_sigpipe_act_error;
+            return true;
+        }
+        return false;
+    case 'i':
+        if (*n ++ == 'g' &&
+            *n ++ == 'n' &&
+            *n ++ == 'o' &&
+            *n ++ == 'r' &&
+            *n ++ == 'e' &&
+            *n == 0) {
+            *t = options_sigpipe_act_ignore;
+            return true;
+        }
+        return false;
+    case 'n':
+        if (*n ++ == 'o' &&
+            *n ++ == '-' &&
+            *n ++ == 'e' &&
+            *n ++ == 'r' &&
+            *n ++ == 'r' &&
+            *n ++ == 'o' &&
+            *n ++ == 'r' &&
+            *n == 0) {
+            *t = options_sigpipe_act_no_error;
+            return true;
+        }
+    }
+    return false;
+}
+
+static enum options_sigpipe_act_t parse_sigpipe_act_optarg(
+    const char* opt_name, const char* opt_arg)
+{
+    enum options_sigpipe_act_t r;
+
+    // stev: short option `-p' means 'no-error'
+    if (opt_arg == NULL)
+        return options_sigpipe_act_no_error;
+
+    if (!lookup_sigpipe_act_name(opt_arg, &r))
+        invalid_opt_arg(opt_name, opt_arg);
+
+    return r;
+}
+
 static size_t parse_num(
     const char* p, const char** q, int b)
 {
-    if (!isdigit(*p)) {
+    if (!ISDIGIT(*p)) {
         *q = p;
         errno = EINVAL;
         return 0;
@@ -1152,7 +1346,7 @@ static size_t parse_debug_optarg(
     if (opt_arg == NULL)
         return 1;
     else {
-        if (!isdigit(opt_arg[0]) || opt_arg[1] != '\0')
+        if (!ISDIGIT(opt_arg[0]) || opt_arg[1] != '\0')
             invalid_opt_arg(opt_name, opt_arg);
         return *opt_arg - '0';
     }
@@ -1196,10 +1390,12 @@ const struct options_t* options(int argc, char* argv[])
         .type_def         = NULL,
         .type_lib         = NULL,
         .type_name        = NULL,
-        .quote_cntrl      = false,
+        .sigpipe_act      = options_sigpipe_act_error,
+        .empty_input      = false,
         .liter_value      = false,
         .surrogate_pairs  = false,
         .valid_utf8       = false,
+        .escape_utf8      = false,
         .ascii_only       = false,
         .multi_obj        = false,
         .print_dots       = false,
@@ -1245,10 +1441,12 @@ const struct options_t* options(int argc, char* argv[])
         type_lib_opt         = 't',
         type_name_opt        = 'N',
         input_file_opt       = 'i',
-        quote_cntrl_opt      = 'q',
+        sigpipe_act_opt      = 'p',
+        empty_input_opt      = 'y',
         liter_value_opt      = 'l',
         surrogate_pairs_opt  = 's',
         valid_utf8_opt       = 'u',
+        escape_utf8_opt      = 'c',
         ascii_only_opt       = 'a',
         multi_obj_opt        = 'm',
         print_dots_opt       = 'o',
@@ -1273,10 +1471,11 @@ const struct options_t* options(int argc, char* argv[])
 #include "opt-sizes.def"
 
         no_type_check_opt,
-        no_quote_cntrl_opt,
+        no_empty_input_opt,
         no_liter_value_opt,
         no_surrogate_pairs_opt,
         no_valid_utf8_opt,
+        no_escape_utf8_opt,
         no_ascii_only_opt,
         no_multi_obj_opt,
         no_print_dots_opt,
@@ -1319,18 +1518,21 @@ const struct options_t* options(int argc, char* argv[])
         { "type-name",              2,       0, type_name_opt },
         { "no-type-check",          0,       0, no_type_check_opt },
         { "input-file",             1,       0, input_file_opt },
-        { "quote-control-chars",    0,       0, quote_cntrl_opt },
-        { "no-quote-control-chars", 0,       0, no_quote_cntrl_opt },
+        { "sigpipe-action",         1,       0, sigpipe_act_opt },
+        { "empty-input",            0,       0, empty_input_opt },
+        { "no-empty-input",         0,       0, no_empty_input_opt },
         { "liter-value",            0,       0, liter_value_opt },
         { "no-liter-value",         0,       0, no_liter_value_opt },
         { "literal-value",          0,       0, liter_value_opt },
         { "no-literal-value",       0,       0, no_liter_value_opt },
         { "valid-utf8",             0,       0, valid_utf8_opt },
-        { "surrogate-pairs",        0,       0, surrogate_pairs_opt },
-        { "no-surrogate-pairs",     0,       0, no_surrogate_pairs_opt },
         { "no-valid-utf8",          0,       0, no_valid_utf8_opt },
         { "validate-utf8",          0,       0, valid_utf8_opt },
         { "no-validate-utf8",       0,       0, no_valid_utf8_opt },
+        { "escape-utf8",            0,       0, escape_utf8_opt },
+        { "no-escape-utf8",         0,       0, no_escape_utf8_opt },
+        { "surrogate-pairs",        0,       0, surrogate_pairs_opt },
+        { "no-surrogate-pairs",     0,       0, no_surrogate_pairs_opt },
         { "ascii-only",             0,       0, ascii_only_opt },
         { "no-ascii-only",          0,       0, no_ascii_only_opt },
         { "multi-object",           0,       0, multi_obj_opt },
@@ -1378,7 +1580,7 @@ const struct options_t* options(int argc, char* argv[])
 #define CASE(type, id, name, pad, letter, desc, help, min, max, def) \
         ARG_LETTER(letter)
 #include "opt-sizes.def"
-        "ad:i:lmN::orqQst:uvV";
+        "acd:i:lmN::oprQst:uvVy";
 
     struct bits_opts_t {
         bits_t dump: 1;
@@ -1395,8 +1597,6 @@ const struct options_t* options(int argc, char* argv[])
 
     int opt;
 
-    lib_version_check();
-
 #define argv_optind()                      \
     ({                                     \
         size_t i = INT_AS_SIZE(optind);    \
@@ -1405,10 +1605,10 @@ const struct options_t* options(int argc, char* argv[])
         argv[i - 1];                       \
     })
 
-#define optopt_char()            \
-    ({                           \
-        ASSERT(isascii(optopt)); \
-        (char) optopt;           \
+#define optopt_char()                   \
+    ({                                  \
+        ASSERT(ISASCII((char) optopt)); \
+        (char) optopt;                  \
     })
 
     opterr = 0;
@@ -1440,11 +1640,15 @@ const struct options_t* options(int argc, char* argv[])
         case type_name_opt:
             opts.type_name = optarg;
             break;
-        case quote_cntrl_opt:
-            opts.quote_cntrl = true;
+        case sigpipe_act_opt:
+            opts.sigpipe_act = parse_sigpipe_act_optarg(
+                "sigpipe-action", optarg);
             break;
-        case no_quote_cntrl_opt:
-            opts.quote_cntrl = false;
+        case empty_input_opt:
+            opts.empty_input = true;
+            break;
+        case no_empty_input_opt:
+            opts.empty_input = false;
             break;
         case liter_value_opt:
             opts.liter_value = true;
@@ -1464,6 +1668,12 @@ const struct options_t* options(int argc, char* argv[])
         case no_valid_utf8_opt:
             opts.valid_utf8 = false;
             break;
+        case escape_utf8_opt:
+            opts.escape_utf8 = true;
+            break;
+        case no_escape_utf8_opt:
+            opts.escape_utf8 = false;
+            break;
         case ascii_only_opt:
             opts.ascii_only = true;
             break;
@@ -1683,6 +1893,10 @@ const struct options_t* options(int argc, char* argv[])
         warning("`-m|--multi-object' is ignored when "
             "given along with `-T|--typelib-func'");
 
+    init_sigpipe_action(opts.sigpipe_act);
+
+    lib_version_check();
+
     return &opts;
 }
 
