@@ -1,4 +1,4 @@
-// Copyright (C) 2016, 2017  Stefan Vargyas
+// Copyright (C) 2016, 2017, 2018  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -80,6 +80,9 @@
     do {} while (0)
 #endif
 
+#if !CONFIG_ERROR_FUNCTION_ATTRIBUTE
+#error we need GCC to support the 'error' function attribute
+#else
 #define STATIC(E)                                   \
     ({                                              \
         extern int __attribute__                    \
@@ -87,6 +90,7 @@
             static_assert();                        \
         (void) ((E) ? 0 : static_assert());         \
     })
+#endif
 
 extern const char stdin_name[];
 
@@ -167,6 +171,13 @@ enum options_type_check_type_t
     options_type_check_lib_type,
 };
 
+enum options_sigpipe_act_t
+{
+    options_sigpipe_act_error,
+    options_sigpipe_act_no_error,
+    options_sigpipe_act_ignore
+};
+
 struct options_t
 {
     const char       *input;
@@ -183,10 +194,14 @@ struct options_t
     const char       *type_lib;
     const char       *type_name;
 
-    bits_t            quote_cntrl: 1;
+    enum options_sigpipe_act_t
+                      sigpipe_act;
+
+    bits_t            empty_input: 1;
     bits_t            liter_value: 1;
     bits_t            surrogate_pairs: 1;
-    bits_t            valid_utf8 : 1;
+    bits_t            valid_utf8: 1;
+    bits_t            escape_utf8: 1;
     bits_t            ascii_only: 1;
     bits_t            multi_obj: 1;
     bits_t            print_dots: 1;
